#include <windows.h>
#include <objbase.h>

#include <map>
#include <stdexcept>

#include "plugin.hpp"

#include "farManager.h"
#include "farEditor.h"
#include "exception.h"
#include "countof.h"
#include "editorContext.h"

void WINAPI SetStartupInfo(const PluginStartupInfo* Info)
{
	farManager = *Info;
	ASSERT_SUCCESS(CoInitialize(0), "Cannot initialize COM");
}

void WINAPI GetPluginInfo(PluginInfo* Info)
{
	static const char pluginName[] = "SSA Editor";
	static const char* const pluginCommands[] = {
		pluginName
	};

	Info->StructSize = sizeof(PluginInfo);
	Info->Flags = PF_DISABLEPANELS | PF_EDITOR;
	Info->DiskMenuStrings = 0;
	Info->DiskMenuNumbers = 0;
	Info->DiskMenuStringsNumber = 0;
	Info->PluginMenuStrings = pluginCommands;
	Info->PluginMenuStringsNumber = COUNTOF(pluginCommands);
	Info->PluginConfigStrings = 0;
	Info->PluginConfigStringsNumber = 0;
	Info->CommandPrefix = 0;
	Info->Reserved = 0;
}

std::map<int, EditorContext> editors;

HANDLE WINAPI OpenPlugin(int OpenFrom, int Item)
{
	try
	{
		editors[GetEditorInfo().EditorID].mainMenu();
	}
	catch (const std::exception& e)
	{
		farManager.showError(e);
	}

	return INVALID_HANDLE_VALUE;
}

void WINAPI ExitFAR()
{
	editors.clear();
	CoUninitialize();
}

int WINAPI ProcessEditorEvent(int Event, void* Param)
{
	try
	{
		switch (Event)
		{
		case EE_CLOSE:
			editors.erase(*reinterpret_cast<int*>(Param));
			break;
		};
	}
	catch (const std::exception& e)
	{
		farManager.showError(e);
	}
	return 0;
}

int WINAPI ProcessEditorInput(const INPUT_RECORD* rec)
{
	try
	{
		return editors[GetEditorInfo().EditorID].processEditorInput(*rec);
	}
	catch (const std::exception& e)
	{
		farManager.showError(e);
	}
	return 0;
}
