#ifndef FAR_MANAGER_INCLUDED
#define FAR_MANAGER_INCLUDED

#include "plugin.hpp"

#include <exception>
#include <string>
#include <sstream>
#include <vector>

class LanguageString
{
public:
	explicit LanguageString(int id);

	const char* get() const;
	operator const char*() const;
private:
	int id_;
};

FarMenuItem menuItem(const char* title, bool selected, bool checked);

const FarMenuItem menuSeparator = {
	"", 0, 0, 1
};

class EnableIf
{
public:
	explicit EnableIf(bool condition);

	void operator()(FarMenuItemEx& item) const;
private:
	bool condition_;
};

class CheckIf
{
public:
	explicit CheckIf(bool condition);

	void operator()(FarMenuItem& item) const;
	void operator()(FarMenuItemEx& item) const;
private:
	bool condition_;
};

class SelectIf
{
public:
	explicit SelectIf(bool condition);

	void operator()(FarMenuItem& item) const;
	void operator()(FarMenuItemEx& item) const;
private:
	bool condition_;
};


class FarMenuItems : public std::vector<FarMenuItem>
{
public:
	FarMenuItems& operator()(const char* title);
	FarMenuItems& operator()(LanguageString title);
	FarMenuItems& operator()();
	template <typename Functor>
	FarMenuItems& operator()(const Functor& functor);
};

class FarMenuItemsEx : public std::vector<FarMenuItemEx>
{
public:
	FarMenuItemsEx& operator()(const char* title);
	FarMenuItemsEx& operator()(LanguageString title);
	FarMenuItemsEx& operator()();
	template <typename Functor>
	FarMenuItemsEx& operator()(const Functor& functor);
};

extern
class FarManager : private PluginStartupInfo
{
public:
	FarManager& operator=(const PluginStartupInfo& other);

	const char* moduleName() const;

	int editorControl(int command, void* param);

	int inputBox(const char* title, const char* prompt, const char* history,
		const char* srcText, char* destText, int destLength,
		const char* help,
		DWORD flags);

	int menu(const FarMenuItem* items, size_t itemCount,
		const char* title = 0,
		const char* bottom = 0,
		const char* help = 0,
		const int* breakKeys = 0,
		int* breakCode = 0);
	int menu(const FarMenuItems& items,
		const char* title = 0,
		const char* bottom = 0,
		const char* help = 0,
		const int* breakKeys = 0,
		int* breakCode = 0);
	int menu(const FarMenuItemEx* items, size_t itemCount,
		const char* title = 0,
		const char* bottom = 0,
		const char* help = 0,
		const int* breakKeys = 0,
		int* breakCode = 0);
	int menu(const FarMenuItemsEx& items,
		const char* title = 0,
		const char* bottom = 0,
		const char* help = 0,
		const int* breakKeys = 0,
		int* breakCode = 0);
	int dialog(FarDialogItem* items, size_t itemCount,
		int width, int height,
		const char* help = 0);
	int dialogEx(FarDialogItem* items, size_t itemCount,
		int width, INT height,
		const char* help,
		FARWINDOWPROC dlgProc);

	long setTextPtr(HANDLE hDlg, int itemId, const char* text);
	long defDlgProc(HANDLE hDlg, int Msg, int Param1, long Param2);

	const char* getMsg(int id) const;

	void showMessage(const char* text, const char* title = "") const;
	void showMessage(const std::string& s, const char* title = "") const;
	void showMessage(const std::ostringstream& oss, const char* title = "") const;
	void showError(const char* text) const;
	void showError(const std::exception& e) const;
} farManager;

#include "farManager.inline.h"

#endif